Option Explicit
'Program: Export.bas for ARIS 6.2
'Purpose: Examines a model and writes a report as successor-table with some infos about the objects in this model.
'         Read-only, that means it shouldn't change the model. Runs as report on one single model.
': Copyright 2007, 2008 Michael Gross, mg()michael-gross;info
'		 This program is free software: you can redistribute it and/or modify
'		 it under the terms of the GNU General Public License as published by
'		 the Free Software Foundation, version 3 of the License.
'
'	   This program is distributed in the hope that it will be useful,
'	   but WITHOUT ANY WARRANTY; without even the implied warranty of
'	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'    GNU General Public License for more details.
'
'    You should have received a copy of the GNU General Public License
'    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'******************************************************************************
'------------------------------------------
'beginn of user-editable variables
'------------------------------------------
'please edit them in Sub Main
Global bOnlyStructureBuildingEdges As Boolean
Global nFilterResultsBy(2) As Byte
Global sUserDefinedStrings(8) As String

Type tTableFormat
	vFontRes As Variant
	vFontType As Variant
End Type
Global vTableFormat(0) As tTableFormat
'------------------------------------------
'end of user-editable variables
'------------------------------------------

Type tOutList
	bActive As Boolean
	bStartObject As Boolean
	bEndObject As Boolean
	bObjectAfterBranch As Boolean
	nOccInModelCount As Long
	oObject As ObjOcc
	sID As String
	sPredIDs As String
	sSuccIDs As String
	sOccInModelList as String
End Type

Global oModel As Object
Global oOutList As New ObjOccList
Global oOutList2() As tOutList

Sub Main
	Dim i As Long
	Dim j As Long
	Dim oModels As Object
	Dim oStartObjects As New ObjOccList
	
	'------------------------------------------
	'beginn of user-editable variables
	'------------------------------------------
	'switch: True: work only with structure edges, False: work with structure and non-structure edges
	'True: nur strukturbildende Kanten verarbeiten, False: strukturbildende und nicht-strukturbildende Kanten verarbeiten
		'There are (1) structure-building and (2) non-structure-building edges.
		'This depends on the definition-type of the objects the edge is linking.
		'example for a structure-building edge: an edge between an funktion and an event
		'example for a non-structure-building edge: an edge between an funktion and an info-object
	bOnlyStructureBuildingEdges = True 'False
		
	'Field 0: Wich delimiter shall be used for lists inside a cell?
	'Field 1: Wich mark shall be used for start-objects?
	'Field 2: Wich mark shall be used for end-objects?
	'Field 3: Wich mark shall be used for objects after branch?
	'Field 4: Wich mark shall be used for inactive objects?
	'Field 5: Wich mark shall be used for active objects?
	
	'Field 6: Do you want to filter the occurrences for columns "Num Occ" and "List Occ" by their group?
	'Yes => Write the name of the valid groups here, No => You can leave this field empty or not.
	
	'Field 7: If you have Microsoft(r) Excel(r) installed and choosed "Excel" as output format: 
	'the worksheet with the output of this script (it's always the 1st one) is named with this string.
	'If you have no Excel(r) installed (maybe OpenOffice instead) leave it empty! 
	'In this case: If you choose "Excel" as output format ARIS will create a text file 
	'(tab separated) - but this string will appear in the next row right in the table!
	
	'All fields can be empty - Do Not delete or add fields
	sUserDefinedStrings(0) = ", "
	sUserDefinedStrings(1) = "A"
	sUserDefinedStrings(2) = "Z"
	sUserDefinedStrings(3) = "X"
	sUserDefinedStrings(4) = "0"
	sUserDefinedStrings(5) = "1"

	sUserDefinedStrings(6) = "Testfall"
	sUserDefinedStrings(7) = "Report"
	
	'set format for output table
	vTableFormat(0).vFontRes = 10
	vTableFormat(0).vFontType = "Arial"
	
	'Field 0: Show models and business cases where the occurrence is active => 2, where the occurrence is inactive => 1, all => 0 
	'Field 1: Show only business cases => 2, only models (not business cases) => 1, all => 0
	'Field 2: Show only models and business cases wich are in special group => 2, all but not in that group => 1, all => 0
	
	nFilterResultsBy(0) = 2
	nFilterResultsBy(1) = 2
	nFilterResultsBy(2) = 2
	'------------------------------------------
	'end of user-editable variables
	'------------------------------------------
	
	Set oModels = SelectedModels
	'for each selected graph
	For i = 0 To oModels.Count -1
		Set oModel = oModels.Get(i)
		Set oOutList = Nothing
		
		If oModel.BuildGraph(bOnlyStructureBuildingEdges) Then	
			'get all start-objects
			Set oStartObjects = oModel.StartNodeList

			'call BFS_execute (all start-objects)
			Call BFS_execute (oStartObjects)
			
			'decide, if the selected model is a businesscase or not
			'selected model is a business case
			If oModel.OriginalModel.IsValid Then
				'call enhance_OutList_bc (OutList)
				Call enhance_OutList_bc()
			'selected model is not a business case
			Else
				'call enhance_OutList (OutList)	
				Call enhance_OutList()
			End If

			'call print_OutList (OutList)
			Call print_OutList()
		End If
	'end for
	Next i
End Sub

Sub BFS_execute (oListOfObjects As Object)
	Dim bCurrentObjectIsInOutList As Boolean
	Dim bEventAfterBranch As Boolean
	Dim bWriteOutList As Boolean
	Dim i As Long
	Dim j As Long
	Dim oCurrentObject As New ObjOcc
	Dim oSuccessorList As New ObjOccList 'all successors of current object
	Dim oYetNotVisitedSuccList As New ObjOccList 'YetNotVisitedSuccList
	
	'***Part 01***
	'get list of objects as 1. parameter
	'decide, if the selected model is a businesscase or not
	'selected model is a business case
	If oModel.OriginalModel.IsValid Then
		'sort list by graphical position from top to down
		oListOfObjects.Sort(SORT_Y, SORT_NONE, SORT_NONE, SelectedLanguage)
	'selected model is not a business case
	Else
		'sort list by graphical position from left to right
		oListOfObjects.Sort(SORT_X, SORT_NONE, SORT_NONE, SelectedLanguage)
	End If
	
	'***Part 02***
	'for each object in list
	For i = 0 To oListOfObjects.Count -1
		Set oCurrentObject = oListOfObjects.Get(i)
		
		'have a look into OutList to see if the current object is already there
		'current object is in OutList
		'current object is not in OutList
		bCurrentObjectIsInOutList = IsObjectInList(oOutList, oCurrentObject)
		If bCurrentObjectIsInOutList = False Then
		
			'write current object in OutList
			oOutList.Add(oCurrentObject)
			'write current object in YetNotVisitedSuccList
			oYetNotVisitedSuccList.Add(oCurrentObject)
			
		End If	
	'end for
	Next i
	
	'***Part 03***
	'for each object in YetNotVisitedSuccList
	For i = 0 To oYetNotVisitedSuccList.Count -1
		Set oCurrentObject = oYetNotVisitedSuccList.Get(i)
		Set oSuccessorList = oModel.GetSuccNodes(oCurrentObject)
    
    'look if current object has successors
		'current object has successors
		If oSuccessorList.Count > 0 Then
			'call BFS_execute (all successors of current object)			
			Call BFS_execute (oSuccessorList)
		End If
		'current object has no successors
	'end for
	Next i
End Sub

Sub enhance_OutList
	Dim bWriteInListOfOccurrences As Boolean
	
	Dim i, j, k As Long
	Dim nID As Long
	
	Dim nNumberOfPredecessors As Long
	Dim nPosPredecessor As Long
	
	Dim nNumberOfSuccessors As Long
	Dim nPosSuccessor As Long
	
	Dim oCurrentPredecessorList As ObjOccList
	Dim oCurrentPredecessor As ObjOcc
	
	Dim oCurrentSuccessorList As ObjOccList
	Dim oCurrentSuccessor As ObjOcc
 
	Dim oOccurrenceList As New ObjOccList     'all occurrences of an object
	Dim oFilteredModelList As New ObjOccList  'contains the filtered list of occurrences
	Dim oFilteredModelList2 As New ModelList  'contains the filtered list of models
  
  Dim oOccurrence As New ObjOcc
	
	Dim sID As String
	Dim sOccInModel As Variant

	'***Part 01***
	'get OutList as 1st parameter
	'move all end-objects in OutList to the end of OutList
	For i=0 To oOutList.Count - 1
		If oModel.GetSuccNodes(oOutList.Get(i)).Count >0 Then
			If i>0 Then
				ReDim Preserve oOutList2(UBound (oOutList2) +1 )
				Set oOutList2(UBound (oOutList2)).oObject = oOutList.Get(i)
			Else
				ReDim Preserve oOutList2(0)
				Set oOutList2(0).oObject = oOutList.Get(i)
			End If
			
			'[initialise oOutList2]
			Call init_OutList()
			
		End If
	Next i
	
	For i=0 To oOutList.Count - 1
		If oModel.GetSuccNodes(oOutList.Get(i)).Count <1 Then
			ReDim Preserve oOutList2(UBound (oOutList2) +1 )
			Set oOutList2(UBound (oOutList2)).oObject = oOutList.Get(i)
			
			'[initialise oOutList2]
			Call init_OutList()
		End If
	Next i
	
	'for all objects in OutList
	nID=1
	For i=0 To UBound(oOutList2)
		'write ID for current object starting with "1"
		oOutList2(i).sID = CStr(nID)
		nID = nID +1
		
		'have a look if current object is a start-object
		'current object is a start-object
		If oModel.GetPredNodes(oOutList2(i).oObject).Count <1 Then
			'mark current object as being a start-object
			oOutList2(i).bStartObject = True
		'current object is not a start-object
		Else
			'mark current object as being not a start-object
			oOutList2(i).bStartObject = False
		End If
		
		'have a look if current object is an end-object
		'current object is an end-object
		If oModel.GetSuccNodes(oOutList2(i).oObject).Count <1 Then
			'mark current object as being an end-object
			oOutList2(i).bEndObject = True
		'current object is not an end-object
		Else
			'mark current object as not being an end-object
			oOutList2(i).bEndObject = False
		End If
		
		'have a look if current object is active
		'current object is active
		If oOutList2(i).oObject.Active Then
			'mark current object as active object
			oOutList2(i).bActive = True
		'current object is not active
		Else
			'mark current object as inactive object
			oOutList2(i).bActive = False
		End If
		
	'end for
	Next i
	
	'***Part 02***
	'for all objects in OutList
	For i=0 To UBound(oOutList2)
	
	  'check object for how many successors it has
		'current object has at least one successor
		nNumberOfSuccessors = oModel.GetSuccNodes(oOutList2(i).oObject).Count
		If nNumberOfSuccessors >0 Then
			
			'for all successors of current object
			For j=0 To nNumberOfSuccessors -1
				Set oCurrentSuccessorList = oModel.GetSuccNodes(oOutList2(i).oObject)
				
				'sort successor by graphical position from left to right
				oCurrentSuccessorList.Sort(SORT_X, SORT_NONE, SORT_NONE, SelectedLanguage)
				Set oCurrentSuccessor = oCurrentSuccessorList.Get(j)
				
				'write ID of successor as SuccID at current object
				sID=""
				nPosSuccessor = 999999
				For k=0 To UBound(oOutList2)
					If oOutList2(k).oObject.IsEqual(oCurrentSuccessor) Then
						sID = oOutList2(k).sID
						nPosSuccessor = k
					End If
				Next k
				
				If oOutList2(i).sSuccIDs <> "" Then
					sID = oOutList2(i).sSuccIDs & sUserDefinedStrings(0) & sID
				End If
				oOutList2(i).sSuccIDs = sID
				
				'check if the current object is on top of a branch
				'current object has more than one successor
				If nNumberOfSuccessors >1 Then
				'mark successor as 'object after branch'
				oOutList2(nPosSuccessor).bObjectAfterBranch = True
				End If
				'current object has only one successor
			'end for
			Next j
		'current object has no successors
		Else
			'write 0 as SuccID at current object
			oOutList2(i).sSuccIDs = "0"
		End If
	'end for
	Next i

	'***Part 03***
	'for all objects in OutList
	For i=0 To UBound(oOutList2)
	
	  'check object for how many predecessors it has
		'current object has at least one predecessor
		nNumberOfPredecessors = oModel.GetPredNodes(oOutList2(i).oObject).Count
		If nNumberOfPredecessors >0 Then
			
			'for all predecessors of current object
			For j=0 To nNumberOfPredecessors -1
				Set oCurrentPredecessorList = oModel.GetPredNodes(oOutList2(i).oObject)
				
				'sort predecessors by graphical position from left to right
				oCurrentPredecessorList.Sort(SORT_X, SORT_NONE, SORT_NONE, SelectedLanguage)
				Set oCurrentPredecessor = oCurrentPredecessorList.Get(j)
				
				'write ID of predecessor as PredID at current object
				sID=""
				nPosPredecessor = 999999
				For k=0 To UBound(oOutList2)
					If oOutList2(k).oObject.IsEqual(oCurrentPredecessor) Then
						sID = oOutList2(k).sID
						nPosPredecessor = k
					End If
				Next k
				
				If oOutList2(i).sPredIDs <> "" Then
					sID = oOutList2(i).sPredIDs & sUserDefinedStrings(0) & sID
				End If
				oOutList2(i).sPredIDs = sID
			
			'end for
			Next j
		'current object has no predecessors
		Else
			'write 0 as PredID at current object
			oOutList2(i).sPredIDs = "0"
		End If
	'end for
	Next i
	
	'***Part 04***
  'get list of user-defined filter-nr. from configuration
  'for all objects in OutList
  For i=0 To UBound(oOutList2)
    Set oOccurrenceList = oOutList2(i).oObject.ObjDef.OccList()
    Set oFilteredModelList = Nothing
    Set oFilteredModelList2 = Nothing
    
    'for all occurrences of current object
    For j=0 To oOccurrenceList.Count -1
		  Set oOccurrence = oOccurrenceList.Get(j)
      bWriteInListOfOccurrences = True
      
      'check if occurrence is active
      'occurrence is active
      If oOccurrence.Active Then
        'check filter.active
        'filter.active = only inactive occurrences
        If nFilterResultsBy(0) = 1 Then
          'don't write this occurrence in ListOfOccurrences
          bWriteInListOfOccurrences = False
        End If
      'occurrence is inactive
      Else
        'filter.active = only active occurrences
        If nFilterResultsBy(0) = 2 Then
          bWriteInListOfOccurrences = False
        End If
      End If
      'write this occurrence in ListOfOccurrences
      
      'check if occurrence is in a business case
      'occurrence is in a business case
      If oOccurrence.OriginalOcc.IsValid Then
        'filter.bc = only occurrences in models
        If nFilterResultsBy(1) = 1 Then
          bWriteInListOfOccurrences = False
        End If
      'occurrence is in a model
      Else
        'filter.bc = only occurrences in business cases
        If nFilterResultsBy(1) = 2 Then
          bWriteInListOfOccurrences = False
        End If
      End If
      
      'check if occurrence is in group ... (filter)
      'occurrence is in group
      If oOccurrence.Model.Group.Name(SelectedLanguage) = sUserDefinedStrings(6) Then
        'filter.group = only occurrences not in group
        If nFilterResultsBy(2) = 1 Then
          bWriteInListOfOccurrences = False
        End If
      'occurrence is not in group
      Else
        'filter.group = only occurrences in group
        If nFilterResultsBy(2) = 2 Then
          bWriteInListOfOccurrences = False
        End If
      End If

      If bWriteInListOfOccurrences Then
      '[store name of model / bc]
        oFilteredModelList.Add(oOccurrence)
      End If
      
    'end for
    Next j
    
    '***Part 05***      
    'sort models / bc in ListOfOccurrences by name
    oFilteredModelList.Sort(SORT_MODELTYPE, SORT_MODEL, SORT_NONE, SelectedLanguage)
    
    'make ListOfOccurrences unique
    '[while sort-method needs an object of type ObjOccList, 
    ' unique-method for modelnames needs an object of type ModelList]
    For j=0 To oFilteredModelList.Count -1
      oFilteredModelList2.Add(oFilteredModelList.Get(j).Model)
    Next j
    oFilteredModelList2.Unique
    
    'count items in ListOfOccurrences
    oOutList2(i).nOccInModelCount = oFilteredModelList2.Count

    '[get name of stored models / bc for current object]
    For j=0 To oFilteredModelList2.Count -1
		sOccInModel = oFilteredModelList2.Get(j).Name(SelectedLanguage)
			If oOutList2(i).sOccInModelList = "" Then
				oOutList2(i).sOccInModelList = sOccInModel
			Else
				oOutList2(i).sOccInModelList = oOutList2(i).sOccInModelList & sUserDefinedStrings(0) & sOccInModel
			End If
	  Next j
  
  'end for
  Next i
End Sub

Sub enhance_OutList_bc
	Dim i, j, k As Long
	Dim nID As Long
	Dim nNumberOfSuccessors As Long
	Dim nPosSuccessor As Long
	
	Dim oCurrentSuccessorList As ObjOccList
	Dim oCurrentSuccessor As ObjOcc
	
	Dim sID As String
	
	'move all endobjects in OutList to the end of OutList
	For i=0 To oOutList.Count - 1

		If i>0 Then
			ReDim Preserve oOutList2(UBound (oOutList2) +1 )
			Set oOutList2(UBound (oOutList2)).oObject = oOutList.Get(i)
		Else
			ReDim Preserve oOutList2(0)
			Set oOutList2(0).oObject = oOutList.Get(i)
		End If
		
		'have a look if current object is active
		'current object is active
		If oOutList2(i).oObject.Active Then
			'mark current object as active
			oOutList2(i).bActive = True
		Else
			'mark current object as inactive
			oOutList2(i).bActive = False
		End If
		
		'[initialise oOutList2]
		Call init_OutList()
		
	'end for
	Next i

End Sub

Sub print_OutList ()
	Dim i, j As Long
	Dim oOutFile As Object
	Dim sTmp As String

	Set oOutFile = New OutputClass
	oOutFile.Init(SelectedFormat, SelectedLanguage)

	oOutFile.BeginTable(100,C_GRAY,COLOR_TRANSPARENT,0,0)
	'print table header
	oOutFile.TableRow()
	oOutFile.TableCell("Name",30,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("DefType",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	
	oOutFile.TableCell("Mark",5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("Activ",5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)

	oOutFile.TableCell("ID",5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("PredIDs",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("SuccIDs",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
	oOutFile.TableCell("Num Occ",6,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("List Occ",30,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	
	oOutFile.TableCell("InEdges Stru",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("OutEdges Stru",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)

	oOutFile.TableCell("InEdges NonStru",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("OutEdges NonStru",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)

	oOutFile.TableCell("SymName",10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("SymNum",5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	
	oOutFile.TableCell("Description",60,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	
	oOutFile.TableCell("ObjDefID",17,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	oOutFile.TableCell("ObjOccID",17,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	
	'print table body
	For i=0 To UBound(oOutList2)
		oOutFile.TableRow()
		
		'Name
		oOutFile.TableCell(oOutList2(i).oObject.ObjDef().Name(SelectedLanguage),30,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'DefType = defintion type name / Name des Definitionstyps
		oOutFile.TableCell(oOutList2(i).oObject.ObjDef().Type(SelectedLanguage),10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
		'Mark = Marker (start-object, end-object, object after branch ...)
		sTmp = ""
		If oOutList2(i).bStartObject Then
			sTmp = sUserDefinedStrings(1)
		ElseIf oOutList2(i).bEndObject Then
			sTmp = sUserDefinedStrings(2)
		End If
		If oOutList2(i).bObjectAfterBranch Then
			sTmp = sUserDefinedStrings(3)
		End If
		oOutFile.TableCell(sTmp,5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
								
		'Activ = Object is active or not / Objekt ist aktiv oder inaktiv
		If oOutList2(i).bActive Then
			sTmp = sUserDefinedStrings(5)
		Else
			sTmp = sUserDefinedStrings(4)
		End If
		oOutFile.TableCell(sTmp,5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
		'ID / ID
		oOutFile.TableCell(oOutList2(i).sID,5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'PredIDs = IDs of predecessors / Vorgaenger - IDs
		oOutFile.TableCell(oOutList2(i).sPredIDs,10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'SuccIDs = IDs of successors / Nachfolger - IDs
		oOutFile.TableCell(oOutList2(i).sSuccIDs,10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)

		'the content of this columns depends on a user-configured filter 
		'/ der Inhalt dieser Spalten hngt von einem beutzer-definierten Filter ab
		'{
			'NumOcc = Number of models or busines cases with occurrences of current object
			'Anzahl der Modelle oder Business Cases mit Ausprgungen dieses Objektes
			oOutFile.TableCell(oOutList2(i).nOccInModelCount,6,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
			'ListOcc = List of models or busines cases with occurrences of current object
			'Liste der Modelle oder Business Cases mit Ausprgungen dieses Objektes
			oOutFile.TableCell(oOutList2(i).sOccInModelList,30,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'}
		
		'InEdgesStru = Number incoming structure edgdes / Anzahl zum Objekt hinfhrenden strukturbildenen Kanten
		oOutFile.TableCell(oOutList2(i).oObject.InEdges(EDGES_STRUCTURE).Count,10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'OutEdgesStru = Number outgoing structure edgdes / Anzahl vom Objekt wegfhrenden strukturbildenen Kanten
		oOutFile.TableCell(oOutList2(i).oObject.OutEdges(EDGES_STRUCTURE).Count,10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
		'InEdgesNonStru = Number incoming non-structure edgdes / Anzahl zum Objekt hinfhrenden nicht-strukturbildenen Kanten
		oOutFile.TableCell(oOutList2(i).oObject.InEdges(EDGES_NONSTRUCTURE).Count,10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'OutEdgesNonStru = Number outgoing non-structure edgdes / Anzahl vom Objekt wegfhrenden nicht-strukturbildenen Kanten
		oOutFile.TableCell(oOutList2(i).oObject.OutEdges(EDGES_NONSTRUCTURE).Count,10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
		'SymName = symbol name / Name des Symbols = Name des Auspraegungsstyps
		oOutFile.TableCell(oOutList2(i).oObject.SymbolName(),10,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'SymNum  = symbol number / Nummer des Symbols = Nummer des Auspraegungsstyps
		oOutFile.TableCell(oOutList2(i).oObject.SymbolNum(),5,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
		'Description - this may cause a failure since it (i'm not 100% sure) depends on the configuration of your database
		'But it is a read-only feature - it should not damage something
		'Beschreibung - dieses Feld verursacht moeglicherweise einen Fehler, weil es (da bin ich mir nicht sicher) von der Konfiguration Ihrer Datenbank abhaengt
		'Aber es ist eine nur-lesen-Operation - sie sollte also nichts kaputt machen
		oOutFile.TableCell(oOutList2(i).oObject.AttrOcc(9).AttrDef(SelectedLanguage).value,60,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)

		'ObjectID = ARIS-internal ObjectID / ARIS - interne Objekt - ID
		'  - "worldwide unique" (IDS Scheer) ID of Object Definition
		oOutFile.TableCell(oOutList2(i).oObject.ObjDef.GUID,17,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		'  - "database unique" (IDS Scheer) ID of Object Occurrence
		oOutFile.TableCell(oOutList2(i).oObject.ObjectID(0),17,vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
		
	Next i
	
	oOutFile.EndTable(sUserDefinedStrings(7),20, vTableFormat(0).vFontType,vTableFormat(0).vFontRes,C_BLACK,COLOR_TRANSPARENT,0,0,0)
	
	'[select filename appendix]
	If SelectedFormat = OUTEXCEL Then
		sTmp = ".xls"
	ElseIf SelectedFormat = OUTHTML Then
		sTmp = ".htm"
	ElseIf SelectedFormat = OUTRTF Then
		sTmp = ".rtf"
	ElseIf SelectedFormat = OUTWORD Then
		sTmp = ".doc"
	ElseIf SelectedFormat = OUTXML Then
		sTmp = ".xml"
	ElseIf SelectedFormat = OUTTABLE Then
		sTmp = ".atf"
	Else
		sTmp = ".txt"
	End If
	'[SelectedFile must be overwrited to avoid a meaningless error message resulting from using a string instead
	'of SelectedFile as 2nd parameter of method WriteReport - although this parameter needs a string]
	SelectedFile = oModel.Name(SelectedLanguage) & sTmp
	oOutFile.WriteReport(SelectedPath, SelectedFile)
End Sub

Function IsObjectInList (oOutList As ObjOccList, oCurrentObject As ObjOcc)
	Dim bCurrentObjectIsInOutList As Boolean
	Dim i As Long

	bCurrentObjectIsInOutList = False
	
	For i = 0 To oOutList.Count - 1
		
		If oOutList.Get(i).IsEqual(oCurrentObject) Then
			bCurrentObjectIsInOutList = True
		End If
	Next i

	IsObjectInList = bCurrentObjectIsInOutList

End Function

Sub init_OutList()
	oOutList2(UBound (oOutList2)).bStartObject = False
	oOutList2(UBound (oOutList2)).bEndObject = False
	oOutList2(UBound (oOutList2)).bObjectAfterBranch = False

	oOutList2(UBound (oOutList2)).nOccInModelCount = 0
			
	oOutList2(UBound (oOutList2)).sID = ""
	oOutList2(UBound (oOutList2)).sSuccIDs = ""
	oOutList2(UBound (oOutList2)).sOccInModelList = ""
End Sub